@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 2005 Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "options.h"
 #include "nutrient.h"
 #include "abbrev.h"
+#include "fdgrp.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,46 +43,52 @@ CA,CU,FE,MG,MN,P,K,SE,NA,ZN
 
 int CarbAminoMap[] =
 {
-32,CHOCDF,FIBTG,STARCH,SUGAR,FRUS,GALS,GLUS,LACS,MALS,SUCS,ALC,
+31,CHOCDF,FIBTG,STARCH,SUGAR,FRUS,GALS,GLUS,LACS,MALS,SUCS,
 PROCNT,ADPROT,ALA_G,ARG_G,ASP_G,CYS_G,GLU_G,GLY_G,HISTN_G,HYP,ILE_G,
 LEU_G,LYS_G,MET_G,PHE_G,PRO_G,SER_G,THR_G,TRP_G,TYR_G,VAL_G
 };
 
 int MiscMap[] =
 {
-24,ENERC_KJ,ASH,WATER,CAFFN,THEBRN,FOLAC,FOLFD,FOLDFE,RETOL,VITA_RAE,
+30,ENERC_KJ,ASH,WATER,CAFFN,THEBRN,ALC,
+FLD,BETN,CHOLN,FOLAC,FOLFD,FOLDFE,RETOL,VITA_RAE,
+VITB12_ADDED,VITE_ADDED,
 TOCPHA,TOCPHB,TOCPHG,TOCPHD,CARTA,CARTB,CRYPX,LUT_ZEA,LYCPN,
 CHOLE,PHYSTR,SITSTR,CAMD5,STID7
 };
 
-int SatMap[] =
+int SatMonoMap[] =
 {
-15,FASAT,F4D0,F6D0,F8D0,F10D0,F12D0,F13D0,F14D0,F15D0,F16D0,
-F17D0,F18D0,F20D0,F22D0,F24D0
+27,FASAT,F4D0,F6D0,F8D0,F10D0,F12D0,F13D0,F14D0,F15D0,F16D0,
+F17D0,F18D0,F20D0,F22D0,F24D0,
+FAMS,F14D1,F15D1,F16D1,F16D1C,F17D1,
+F18D1,F18D1C,F20D1,F22D1,F22D1C,F24D1C
 };
 
-int UnsatMap[] =
+int PolyTransMap[] =
 {
-34,FAMS,F14D1,F15D1,F16D1,F16D1C,F17D1,F18D1,F18D1C,F20D1,F22D1,F22D1C,F24D1C,
-FAPU,F18D2,F18D2CN6,F18D3,F18D3CN3,F18D3CN6,F18D4,F20D2CN6,F20D3,
-F20D4,F20D5,F22D5,F22D6,
-FATRN,FATRNM,F16D1T,F18D1T,FATRNP,F18D2T,F18D2I,F18D2CLA,F18D3I
+29,FAPU,F18D2,F18D2CN6,F18D3,F18D3CN3,F18D3CN6,F18D4,F20D2CN6,F20D3,
+F20D3N3,F20D3N6,
+F20D4,F20D4N6,F20D5,F21D5,F22D4,F22D5,F22D6,
+FATRN,FATRNM,F16D1T,F18D1T,F22D1T,FATRNP,F18D2T,F18D2TT,F18D2I,F18D2CLA,F18D3I
 };
 
 int *ScreenMap[] =
 {
-DVMap,DVMap,CarbAminoMap,MiscMap,SatMap,UnsatMap
+DVMap,DVMap,CarbAminoMap,MiscMap,SatMonoMap,PolyTransMap
 };
 
 char *ScreenTitle[] =
 {
 "Daily Value Percentages","Daily Value Absolute Amounts",
 "Carbohydrates and Amino Acids","Miscellaneous Nutrients",
-"Saturated Fatty Acids","Unsaturated Fatty Acids"
+"Saturated and Monounsaturated Fatty Acids","Polyunsaturated and Trans Fatty Acids"
 };
 
 int MaxScreen = 6;
 
+int MaxFdGrp = FD_GRP_COUNT;
+
 float DVBase[NUTRIENT_COUNT];
 
 float DV[NUTRIENT_COUNT];
@@ -97,6 +104,7 @@ void initializations()
 {
 int c;
 NoData = atof("-0");
+test_signsense(&NoData);
 for (c=0; c < NUTRIENT_COUNT; c++) DVBase[c] = 0;
 
 DVBase[ENERC_KCAL] = 2000;
@@ -129,12 +137,12 @@ DVBase[VITC] = 60;
 DVBase[FASAT] = 20;
 DVBase[CHOLE] = 300;
 DVBase[VITD] = 400;
-DVBase[LA] = 10;
-DVBase[AA] = .18;
+DVBase[LA] = 4.44;
+DVBase[AA] = 0.2;
 DVBase[OMEGA6] = DVBase[LA] + DVBase[AA];
 DVBase[ALA] = 2.22;
-DVBase[EPA] = 0.36;
-DVBase[DHA] = 0.36;
+DVBase[EPA] = 0.33;
+DVBase[DHA] = 0.33;
 DVBase[OMEGA3] = DVBase[ALA] + DVBase[EPA] + DVBase[DHA];
 DVBase[FAPU] = DVBase[OMEGA6] + DVBase[OMEGA3];
 DVBase[FAMS] = 61.5 - DVBase[FASAT] - DVBase[FAPU];
@@ -508,7 +516,7 @@ char servingstring[62];
 int count;
 printf("%-60s   Grams  %8.2f\n",food_ptr->name,food_ptr->grams * *ratio);
 printf("Serving:  %-50s   Ounces  %7.2f\n",format_serving(servingstring,ratio,food_ptr),*ratio * food_ptr->grams / 28.35);
-printf("          %-50s   Water    %6.2f%%\n"," ",food_ptr->nutrient[WATER]);
+printf("          %-50s   Water    %6.2f%%\n"," ",food_ptr->nutrient[WATER] > 0 ? food_ptr->nutrient[WATER] : 0.0);
 if ( ! options.custom && options.screen == 0 ) printf("Percentages of \"Daily Values\" in this serving:                 Refuse   %3d%%\n\n",food_ptr->refuse);
 if ( options.custom && options.screen == 0 )     printf("Percentages of customized DV in this serving:                  Refuse   %3d%%\n\n",food_ptr->refuse);
 if ( options.screen > 0 )                 printf("Nutrients in this serving:                                     Refuse   %3d%%\n\n",food_ptr->refuse);
@@ -583,14 +591,21 @@ void food_display(FILE *fp)
 {
 int i;
 float pctfat = 0, pctcarb = 0, pctprot = 0;
+float p3, p6, h3, h6, o;
 char carbprotfat[9];
-if (food_work.nutrient[ENERC_KCAL] > 0)
+if (food_work.nutrient[ENERC_KCAL] > 0.05)
  {
  pctcarb = 100 * food_work.nutrient[CHO_KCAL] / food_work.nutrient[ENERC_KCAL];
  pctprot = 100 * food_work.nutrient[PROT_KCAL] / food_work.nutrient[ENERC_KCAL]; 
  pctfat = 100 * food_work.nutrient[FAT_KCAL] / food_work.nutrient[ENERC_KCAL];
+ if (test_for_negative_zero(&pctcarb)) pctcarb = 0;
+ if (test_for_negative_zero(&pctprot)) pctprot = 0;
+ if (test_for_negative_zero(&pctfat)) pctfat = 0;
+ }
+else
+ {
+ pctcarb = 0; pctprot = 0; pctfat = 0;
  }
-
 sprintf(carbprotfat,"%1.0f/%1.0f/%1.0f",pctcarb,pctprot,pctfat);
 switch (options.screen)
  {
@@ -632,86 +647,93 @@ switch (options.screen)
  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,4);
  food_display_line(fp,ScreenMap[options.screen][i+16],0,1,0,0);
  food_display_line(fp,ScreenMap[options.screen][13],0,0,0,0);
- food_display_line(fp,ScreenMap[options.screen][13+16],0,1,0,0);
+ if (!options.n6hufaspec) food_display_line(fp,ScreenMap[options.screen][13+16],0,1,0,0);
+ if (options.n6hufaspec)
+  {
+  food_display_line(fp,ScreenMap[options.screen][13+16],0,0,0,0);
+  p3 = 902 * food_work.nutrient[SHORT3] / food_work.nutrient[ENERC_KCAL];
+  p6 = 902 * food_work.nutrient[SHORT6] / food_work.nutrient[ENERC_KCAL];
+  h3 = 902 * food_work.nutrient[LONG3] / food_work.nutrient[ENERC_KCAL];
+  h6 = 902 * food_work.nutrient[LONG6] / food_work.nutrient[ENERC_KCAL];
+  o  = 902 * food_work.nutrient[FAT] * options.fattyacidfactor / food_work.nutrient[ENERC_KCAL] - food_work.nutrient[SHORT6] - food_work.nutrient[LONG6] - food_work.nutrient[SHORT3] - food_work.nutrient[LONG3];
+  fprintf(fp,"Omega-6 HUFA     %5.1f%%\n",n6hufa(p3,p6,h3,h6,o));
+  }
  break;
  case 2 :
  fprintf(fp,"%-27s","Carbohydrates");
  fprintf(fp,"%-27s\n\n","Amino Acids");
- for (i = 1; i < 11 ; i++)
+ for (i = 1; i < 5 ; i++)
   {
   food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+11],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+22],0,1,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+10],0,0,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+21],0,1,0,0);
   }
- food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
- food_display_line(fp,ScreenMap[options.screen][i+11],0,1,0,0);
+ for (i = 5; i < 11 ; i++)
+  {
+  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,1);
+  food_display_line(fp,ScreenMap[options.screen][i+10],0,0,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+21],0,1,0,0);
+  }
+ fprintf(fp,"%-27s"," ");
+ food_display_line(fp,ScreenMap[options.screen][i+10],0,1,0,0);
  fprintf(fp,"\n\n");
  break;
  case 3 :
  fprintf(fp,"Miscellaneous Nutrients\n\n");
- for (i = 1; i < 6 ; i++)
+ for (i = 1; i < 11 ; i++)
   {
   food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
   food_display_line(fp,ScreenMap[options.screen][i+10],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+19],0,1,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+20],0,1,0,0);
   }
- for (i = 6; i < 10 ; i++)
-  {
-  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+10],0,1,0,0);
-  }
- food_display_line(fp,ScreenMap[options.screen][i],0,1,0,0);
  fprintf(fp,"\n\n\n");
  break;
  case 4 :
- fprintf(fp,"Saturated Fatty Acids\n\n");
- for (i = 1; i < 8 ; i++)
+ food_display_line(fp,ScreenMap[options.screen][1],0,0,1,0);
+ fprintf(fp,"Saturated and Monounsaturated Fatty Acids\n");
+ food_display_line(fp,ScreenMap[options.screen][2],0,1,1,0);
+ for (i = 3; i < 15 ; i++)
   {
-  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+8],0,1,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+13],0,1,1,0);
   }
- food_display_line(fp,ScreenMap[options.screen][i],0,1,0,0);
- fprintf(fp,"\n\n\n\n\n");
+ food_display_line(fp,ScreenMap[options.screen][i],0,1,1,0);
  break;
  case 5 :
- fprintf(fp,"%-27s","Unsaturated Fatty Acids");
- food_display_line(fp,ScreenMap[options.screen][13],0,0,1,0);
- food_display_line(fp,ScreenMap[options.screen][26],0,1,1,0);
- fprintf(fp,"%-27s"," ");
- food_display_line(fp,ScreenMap[options.screen][14],0,0,1,0);
- food_display_line(fp,ScreenMap[options.screen][27],0,1,1,0);
- for (i = 1; i < 8 ; i++)
+ fprintf(fp,"%-27s","Polyunsaturated and Trans Fatty Acids\n\n");
+ for (i = 1; i < 10 ; i++)
   {
   food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
-  food_display_line(fp,ScreenMap[options.screen][i+14],0,0,1,0);
-  food_display_line(fp,ScreenMap[options.screen][i+27],0,1,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+9],0,0,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+18],0,1,1,0);
   }
- for (i = 8; i < 12 ; i++)
+ for (i = 10; i < 12 ; i++)
   {
-  food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
-  food_display_line(fp,ScreenMap[options.screen][i+14],0,1,1,0);
+  fprintf(fp,"%-54s"," ");
+  food_display_line(fp,ScreenMap[options.screen][i+18],0,1,1,0);
   }
- food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
  fprintf(fp,"\n\n");
  break;
  default :
  if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][1]])) fprintf(fp,"Calories(%4.0f)   %5.0f%%     ",DV[ScreenMap[options.screen][1]],food_work.nutrient[ScreenMap[options.screen][1]]/DV[ScreenMap[options.screen][1]]*100);
  else fprintf(fp,"Calories(%4.0f)    %5s     ",DV[ScreenMap[options.screen][1]],"(nd)");
- food_display_line(fp,ScreenMap[options.screen][14],1,0,0,0);
- food_display_line(fp,ScreenMap[options.screen][17],1,1,0,0);
+ if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][14]])) fprintf(fp,"%-14s   %5.0f%%     ",Nutrient[ScreenMap[options.screen][14]],dv_cho());
+ else fprintf(fp,"%-14s    %5s     ",Nutrient[ScreenMap[options.screen][14]],"(nd)");
+ if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][17]])) fprintf(fp,"%-14s   %5.0f%%\n",Nutrient[ScreenMap[options.screen][17]],dv_pro());
+ else fprintf(fp,"%-14s    %5s\n",Nutrient[ScreenMap[options.screen][17]],"(nd)");
  fprintf(fp,"Carb/Prot/Fat  %8s     ",carbprotfat);
  food_display_line(fp,ScreenMap[options.screen][15],1,0,0,0);
  if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][16]])) fprintf(fp,"%-15s  %5.0f%-1s\n",Nutrient[ScreenMap[options.screen][16]],food_work.nutrient[ScreenMap[options.screen][16]],Unit[ScreenMap[options.screen][16]]);
  else fprintf(fp,"%-15s   %5s\n",Nutrient[ScreenMap[options.screen][16]],"(nd)");
  fprintf(fp,"\n");
- if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][2]])) fprintf(fp,"%-14s   %5.0f%%     ",Nutrient[ScreenMap[options.screen][2]],food_work.nutrient[ScreenMap[options.screen][2]]/dv_fat()*100);
+ if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][2]])) fprintf(fp,"%-14s   %5.0f%%     ",Nutrient[ScreenMap[options.screen][2]],dv_fat());
  else fprintf(fp,"%-14s    %5s     ",Nutrient[ScreenMap[options.screen][2]],"(nd)");
  food_display_line(fp,ScreenMap[options.screen][18],1,0,0,0);
  food_display_line(fp,ScreenMap[options.screen][30],1,1,0,0);
  food_display_line(fp,ScreenMap[options.screen][3],1,0,0,1);
  food_display_line(fp,ScreenMap[options.screen][19],1,0,0,0);
  food_display_line(fp,ScreenMap[options.screen][31],1,1,0,0);
- if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][4]])) fprintf(fp," %-13s   %5.0f%%     ",Nutrient[ScreenMap[options.screen][4]],food_work.nutrient[ScreenMap[options.screen][4]]/dv_fams()*100);
+ if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][4]])) fprintf(fp," %-13s   %5.0f%%     ",Nutrient[ScreenMap[options.screen][4]],food_work.nutrient[ScreenMap[options.screen][4]]/DV[FAMS]*100);
  else fprintf(fp," %-13s    %5s     ",Nutrient[ScreenMap[options.screen][4]],"(nd)");
  food_display_line(fp,ScreenMap[options.screen][20],1,0,0,0);
  food_display_line(fp,ScreenMap[options.screen][32],1,1,0,0);
@@ -739,7 +761,17 @@ switch (options.screen)
  food_display_line(fp,ScreenMap[options.screen][i],1,0,0,4);
  food_display_line(fp,ScreenMap[options.screen][i+16],1,1,0,0);
  food_display_line(fp,ScreenMap[options.screen][13],1,0,0,0);
- food_display_line(fp,ScreenMap[options.screen][13+16],1,1,0,0);
+ if (!options.n6hufaspec) food_display_line(fp,ScreenMap[options.screen][13+16],1,1,0,0);
+ if (options.n6hufaspec)
+  {
+  food_display_line(fp,ScreenMap[options.screen][13+16],1,0,0,0);
+  p3 = 902 * food_work.nutrient[SHORT3] / food_work.nutrient[ENERC_KCAL];
+  p6 = 902 * food_work.nutrient[SHORT6] / food_work.nutrient[ENERC_KCAL];
+  h3 = 902 * food_work.nutrient[LONG3] / food_work.nutrient[ENERC_KCAL];
+  h6 = 902 * food_work.nutrient[LONG6] / food_work.nutrient[ENERC_KCAL];
+  o  = 902 * food_work.nutrient[FAT] * options.fattyacidfactor / food_work.nutrient[ENERC_KCAL] - food_work.nutrient[SHORT6] - food_work.nutrient[LONG6] - food_work.nutrient[SHORT3] - food_work.nutrient[LONG3];
+  fprintf(fp,"Omega-6 HUFA     %5.0f%%\n",n6hufa(p3,p6,h3,h6,o));
+  }
  break;
  }
 }
@@ -775,7 +807,7 @@ switch (flags)
   if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%8.1f %-3s\n",food_work.nutrient[nut],Unit[nut]);
   else fprintf(fp,"%9s%-3s\n","(nd)"," ");
 #else
-  if (nut != VITE && nut != VITK && nut != ILE_G && nut != CARTB && nut != CRYPX && nut != LUT_ZEA && nut != LYCPN && nut != VITA_RAE && nut != F14D0)
+  if (nut != VITE && nut != VITK && nut != ILE_G)
    {
    if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%7.1f %-3s\n",food_work.nutrient[nut],Unit[nut]);
    else fprintf(fp,"%8s%-3s\n","(nd)"," ");
@@ -792,7 +824,7 @@ switch (flags)
   if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%8.2f %-3s\n",food_work.nutrient[nut],Unit[nut]);
   else fprintf(fp,"%9s%-3s\n","(nd)"," ");
 #else
-  if (nut != F20D2CN6 && nut != F20D3 && nut != F20D4 && nut != F20D5 && nut != F22D5 && nut != F22D6)
+  if (nut != F4D0 && nut != F18D2CN6 && nut != F20D2CN6 && nut != F20D3 && nut != F20D4 && nut != F20D5 && nut != F22D4 && nut != F24D0)
    {
    if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%7.2f %-3s\n",food_work.nutrient[nut],Unit[nut]);
    else fprintf(fp,"%8s%-3s\n","(nd)"," ");
@@ -823,18 +855,24 @@ new_recipe->next = NULL;
 food_ptr->next = new_recipe;
 }
 
-void delete_recipe_food(int num)
+void modify_recipe_food(int num, char *qty)
 {
 struct food *m, *recipe_ptr = &recipe_root;
 int count = 0;
+float newqty;
 while (recipe_ptr->next != NULL)
  {
  count++;
  if (count == num)
   {
-  m = recipe_ptr->next;
-  recipe_ptr->next = recipe_ptr->next->next;
-  free(m);
+  newqty = evaluate_qty(food_number(recipe_ptr->next->food_no),qty);
+  if (newqty == 0)
+   {
+   m = recipe_ptr->next;
+   recipe_ptr->next = recipe_ptr->next->next;
+   free(m);
+   }
+  else recipe_ptr->next->grams = newqty;
   return;
   }
  recipe_ptr = recipe_ptr->next;
@@ -845,26 +883,36 @@ void compute_derived_fields(struct food *food_ptr)
 {
 float ratio;
 food_ptr->nutrient[VITE] = 1.5 * food_ptr->nutrient[TOCPHA];
-food_ptr->nutrient[LA] = food_ptr->nutrient[F18D2CN6] > 0 ? food_ptr->nutrient[F18D2CN6] : food_ptr->nutrient[F18D2] - food_ptr->nutrient[F18D2T] - food_ptr->nutrient[F18D2I] - food_ptr->nutrient[F18D2CLA];
+food_ptr->nutrient[LA] = food_ptr->nutrient[F18D2CN6] > 0 ? food_ptr->nutrient[F18D2CN6] : food_ptr->nutrient[F18D2] - food_ptr->nutrient[F18D2T] - food_ptr->nutrient[F18D2TT] - food_ptr->nutrient[F18D2I] - food_ptr->nutrient[F18D2CLA];
 if (food_ptr->nutrient[LA] == 0 && test_for_negative_zero(&food_ptr->nutrient[F18D2])) food_ptr->nutrient[LA] = NoData;
 if (food_ptr->nutrient[LA] < 0) food_ptr->nutrient[LA] = 0;
-food_ptr->nutrient[OMEGA6] = food_ptr->nutrient[LA];
+food_ptr->nutrient[SHORT6] = food_ptr->nutrient[LA];
+food_ptr->nutrient[SHORT6] += food_ptr->nutrient[F18D3CN6];
 food_ptr->nutrient[ALA] = food_ptr->nutrient[F18D3CN3] > 0 ? food_ptr->nutrient[F18D3CN3] : food_ptr->nutrient[F18D3] - food_ptr->nutrient[F18D3CN6] - food_ptr->nutrient[F18D3I];
 if (food_ptr->nutrient[ALA] == 0 && test_for_negative_zero(&food_ptr->nutrient[F18D3])) food_ptr->nutrient[ALA] = NoData;
 if (food_ptr->nutrient[ALA] < 0) food_ptr->nutrient[ALA] = 0;
-food_ptr->nutrient[OMEGA3] = food_ptr->nutrient[ALA];
-food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F18D4];
-food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D2CN6];
-food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D3];
-food_ptr->nutrient[OMEGA6] += food_ptr->nutrient[F20D4];
+food_ptr->nutrient[SHORT3] = food_ptr->nutrient[ALA];
+food_ptr->nutrient[SHORT3] += food_ptr->nutrient[F18D4];
+food_ptr->nutrient[LONG6] = food_ptr->nutrient[F20D2CN6];
+if (food_ptr->nutrient[F20D3N3] > 0) food_ptr->nutrient[LONG3] += food_ptr->nutrient[F20D3N3];
+if (food_ptr->nutrient[F20D3N6] > 0) food_ptr->nutrient[LONG6] += food_ptr->nutrient[F20D3N6];
+if (food_ptr->nutrient[F20D3N3] == 0 && food_ptr->nutrient[F20D3N6] == 0) food_ptr->nutrient[LONG6] += food_ptr->nutrient[F20D3];
 food_ptr->nutrient[AA] = food_ptr->nutrient[F20D4];
-food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F20D5];
+if (food_ptr->nutrient[F20D4N6] > 0) food_ptr->nutrient[AA] = food_ptr->nutrient[F20D4N6];
+food_ptr->nutrient[LONG6] += food_ptr->nutrient[AA];
 food_ptr->nutrient[EPA] = food_ptr->nutrient[F20D5];
-food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F22D5];
-food_ptr->nutrient[OMEGA3] += food_ptr->nutrient[F22D6];
+food_ptr->nutrient[LONG3] = food_ptr->nutrient[F20D5];
+food_ptr->nutrient[LONG3] += food_ptr->nutrient[F22D5];
 food_ptr->nutrient[DHA] = food_ptr->nutrient[F22D6];
-if (food_ptr->nutrient[OMEGA3] == 0 && test_for_negative_zero(&food_ptr->nutrient[ALA]) && test_for_negative_zero(&food_ptr->nutrient[EPA]) && test_for_negative_zero(&food_ptr->nutrient[DHA])) food_ptr->nutrient[OMEGA3] = NoData;
-if (food_ptr->nutrient[OMEGA6] == 0 && test_for_negative_zero(&food_ptr->nutrient[LA]) && test_for_negative_zero(&food_ptr->nutrient[AA])) food_ptr->nutrient[OMEGA6] = NoData;
+food_ptr->nutrient[LONG3] += food_ptr->nutrient[F22D6];
+if (food_ptr->nutrient[SHORT3] == 0 && test_for_negative_zero(&food_ptr->nutrient[ALA]) && test_for_negative_zero(&food_ptr->nutrient[F18D4])) food_ptr->nutrient[SHORT3] = NoData;
+if (food_ptr->nutrient[LONG3] == 0 && test_for_negative_zero(&food_ptr->nutrient[EPA]) && test_for_negative_zero(&food_ptr->nutrient[F22D5]) && test_for_negative_zero(&food_ptr->nutrient[DHA])) food_ptr->nutrient[LONG3] = NoData;
+if (food_ptr->nutrient[SHORT6] == 0 && test_for_negative_zero(&food_ptr->nutrient[LA]) && test_for_negative_zero(&food_ptr->nutrient[F18D3CN6])) food_ptr->nutrient[SHORT6] = NoData;
+if (food_ptr->nutrient[LONG6] == 0 && test_for_negative_zero(&food_ptr->nutrient[F20D2CN6]) && test_for_negative_zero(&food_ptr->nutrient[F20D3]) && test_for_negative_zero(&food_ptr->nutrient[F20D3N6]) && test_for_negative_zero(&food_ptr->nutrient[AA])) food_ptr->nutrient[LONG6] = NoData;
+food_ptr->nutrient[OMEGA6] = food_ptr->nutrient[SHORT6] + food_ptr->nutrient[LONG6];
+if (food_ptr->nutrient[OMEGA6] == 0 && test_for_negative_zero(&food_ptr->nutrient[SHORT6]) && test_for_negative_zero(&food_ptr->nutrient[LONG6])) food_ptr->nutrient[OMEGA6] = NoData;
+food_ptr->nutrient[OMEGA3] = food_ptr->nutrient[SHORT3] + food_ptr->nutrient[LONG3];
+if (food_ptr->nutrient[OMEGA3] == 0 && test_for_negative_zero(&food_ptr->nutrient[SHORT3]) && test_for_negative_zero(&food_ptr->nutrient[LONG3])) food_ptr->nutrient[OMEGA3] = NoData;
 
 if ( food_ptr->prot_cal_factor == 0 && food_ptr->fat_cal_factor == 0 && food_ptr->cho_cal_factor == 0 && food_ptr->nutrient[ENERC_KCAL] > 0)
  {
@@ -884,31 +932,34 @@ else
  }
 food_ptr->nutrient[CHO_NONFIB] = food_ptr->nutrient[CHOCDF] - food_ptr->nutrient[FIBTG];
 if (food_ptr->nutrient[CHO_NONFIB] < 0) food_ptr->nutrient[CHO_NONFIB] = 0;
-if (food_ptr->nutrient[FIBTG] == 0 && test_for_negative_zero(&food_ptr->nutrient[FIBTG])) food_ptr->nutrient[CHO_NONFIB] = NoData;
-}
-
-float dv_fams()
-{
-float targetpct, totalfa, totaldv, imaginarydv, diff;
-targetpct = food_work.nutrient[FAT] / dv_fat();
-totalfa = food_work.nutrient[FASAT] + food_work.nutrient[FAMS] + food_work.nutrient[FAPU];
-totaldv = DV[FASAT] + DV[FAMS] + DV[FAPU];
-imaginarydv = totalfa / targetpct;
-diff = totaldv - imaginarydv;
-if (DV[FAMS] > diff) return DV[FAMS] - diff;
-else return DV[FAMS];
 }
 
 float dv_fat()
 {
 float targetpct, totalcal, totaldv, imaginarydv, diff;
 targetpct = food_work.nutrient[ENERC_KCAL] / DV[ENERC_KCAL];
-totalcal = food_work.nutrient[PROCNT] * options.protcalpergm + food_work.nutrient[CHOCDF] * options.carbcalpergm + food_work.nutrient[FAT] * options.fatcalpergm;
-totaldv = DV[PROCNT] * options.protcalpergm + DV[CHOCDF] * options.carbcalpergm + DV[FAT] * options.fatcalpergm;
-imaginarydv = totalcal / targetpct;
-diff = (totaldv - imaginarydv) / options.fatcalpergm;
-if (DV[FAT] > diff) return DV[FAT] - diff;
-else return DV[FAT];
+if (options.pccarb == 0 && options.pcprotein == 0)
+ {
+ totalcal = food_work.nutrient[PROCNT] * options.protcalpergm + food_work.nutrient[CHOCDF] * options.carbcalpergm + food_work.nutrient[FAT] * options.fatcalpergm;
+ totaldv = DV[PROCNT] * options.protcalpergm + DV[CHOCDF] * options.carbcalpergm + DV[FAT] * options.fatcalpergm;
+ imaginarydv = totalcal / targetpct;
+ diff = (totaldv - imaginarydv) / options.fatcalpergm;
+ if (DV[FAT] > diff) return food_work.nutrient[FAT] / (DV[FAT] - diff) * 100;
+ else return food_work.nutrient[FAT] / DV[FAT] * 100;
+ }
+else return diff = 100 * food_work.nutrient[FAT_KCAL] / (DV[ENERC_KCAL] * (1 - (options.pccarb > 0 ? options.pccarb / 100 : DV[CHOCDF] * options.carbcalpergm / DV[ENERC_KCAL]) - (options.pcprotein > 0 ? options.pcprotein / 100 : DV[PROCNT] * options.protcalpergm / DV[ENERC_KCAL])));
+}
+
+float dv_cho()
+{
+if (options.pccarb > 0) return food_work.nutrient[CHO_KCAL] / DV[ENERC_KCAL] / options.pccarb * 10000;
+else return food_work.nutrient[CHOCDF] / DV[CHOCDF] * 100;
+}
+
+float dv_pro()
+{
+if (options.pcprotein > 0) return food_work.nutrient[PROT_KCAL] / DV[ENERC_KCAL] / options.pcprotein * 10000;
+else return food_work.nutrient[PROCNT] / DV[PROCNT] * 100;
 }
 
 char *format_serving(char *buffer, float *ratio, struct food *food_ptr)
@@ -919,3 +970,17 @@ strncat(buffer,food_ptr->unit,50);
 buffer[49] = '\0';
 return buffer;
 }
+
+float n6hufa(float p3, float p6, float h3, float h6, float o)
+{
+float pc6 = .0441, pc3 = .0555, co = 5, ks = .175, hi3 = .005, hc6 = .7, hc3 = 3;
+float answer;
+/*
+printf("%f %f %f %f %f\n",p3,p6,h3,h6,o);
+exit(0);
+*/  
+answer = 100 / (1 + pc6/p6 * (1 + p3/pc3 + h3/hi3 + o/co + p6/ks)) + 100 / (1 + hc6/h6 * (1 + h3/hc3));
+if (answer <= 90 && answer >= 15) return answer;
+if (answer > 90) return 90;
+else return 15;
+}
