@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 2005 Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <math.h>
 
 struct opt options;
 
@@ -127,12 +126,14 @@ float temp, lastsetting;
 lastsetting = options.abnuts[FAPU];
 header("NUT:  Essential Fats Absolute Amount");
 spacer(0);
+options.n6hufaspec = 0;
 printf("\nEnter absolute amount of EFAs in grams (just <enter> for default):  ");
 get_string(buf,127);
 options.abnuts[FAPU] = atof(buf);
 options.locknuts[FAPU] = 1;
 if ( options.abnuts[FAPU] < 0 ) 
  {
+ if (options.abnuts[FAPU] == -1) options.n6hufaspec = 1;
  options.abnuts[FAPU] = 0;
  options.locknuts[FAPU] = 0;
  }
@@ -224,6 +225,7 @@ if ( options.fatprotratio > 0 )
   options.abnuts[CHOCDF] = 0;
   options.locknuts[CHOCDF] = 0;
   }
+ else if (options.abnuts[CHOCDF] == 0 && options.pccarb == 0) options.pcprotein = 30 / options.fatprotratio;
  }
 options.abnuts[FAT] = 0;
 options.locknuts[FAT] = 0;
@@ -361,6 +363,14 @@ options.screen %= MaxScreen;
 write_OPTIONS();
 }
 
+void screen_previous(void)
+{
+    if (--options.screen < 0) {
+        options.screen = MaxScreen - 1;
+    }
+    write_OPTIONS();
+}
+
 void get_cals(float *cals)
 {
 char buff[128];
@@ -370,7 +380,7 @@ fgets(buff,128,stdin);
 
 void auto_cal(float *bufptr, int analflag)
 {
-float buf = *bufptr, temp = 0, adjaa = .5, adjsat = .9, adjla = .9;
+float buf = *bufptr, temp = 0, satfatpct = .39;
 int i;
 if (options.autocal) buf = average_cals(analflag);
 if (analflag)
@@ -395,13 +405,16 @@ if (buf != 1)
  {
  DV[PROCNT] = buf * DVBase[PROCNT] * 4 / options.protcalpergm;
  DV[FAT] = buf * DVBase[FAT] * 9.2307692 / options.fatcalpergm;
- DV[CHOCDF] = buf * DVBase[CHOCDF] * 4 / options.carbcalpergm;;
+ DV[CHOCDF] = buf * DVBase[CHOCDF] * 4 / options.carbcalpergm;
  }
 if (buf == 1)
  {
  DV[PROCNT] = DVBase[PROCNT];
  DV[FAT] = DVBase[FAT];
  DV[CHOCDF] = DVBase[CHOCDF];
+ options.protcalpergm = 4;
+ options.carbcalpergm = 4;
+ options.fatcalpergm = 9.2307692;
  }
 DV[FIBTG] = buf * DVBase[FIBTG];
 DV[FASAT] = buf * DVBase[FASAT];
@@ -433,12 +446,13 @@ if (options.fatprotratio > 0 && (options.abnuts[PROCNT] > 0 || options.pcprotein
  temp = 1 - (DV[PROCNT] * options.protcalpergm + DV[FAT] * options.fatcalpergm) / DV[ENERC_KCAL];
  DV[CHOCDF] = temp * DV[ENERC_KCAL] / options.carbcalpergm;
  }
-if (options.fatprotratio <= 0)
+if (options.fatprotratio <= 0 && (DV[PROCNT] != DVBase[PROCNT] || DV[CHOCDF] != DVBase[CHOCDF]))
  {
  temp = 1 - (DV[PROCNT] * options.protcalpergm + DV[CHOCDF] * options.carbcalpergm) / DV[ENERC_KCAL];
  DV[FAT] = temp * DV[ENERC_KCAL] / options.fatcalpergm;
  }
 DV[FAMS] = options.fattyacidfactor * DV[FAT] - DV[FASAT] - DV[FAPU];
+if (analflag && options.abnuts[FAPU] == 0) efa_dynamics();
 if (options.abnuts[FAPU] > 0)
  {
  DV[FAMS] = DV[FAMS] - options.abnuts[FAPU] + DV[FAPU];
@@ -451,6 +465,19 @@ if (options.abnuts[FAPU] > 0)
  if (options.abnuts[EPA] > 0) DV[EPA] = options.abnuts[EPA];
  if (options.abnuts[DHA] > 0) DV[DHA] = options.abnuts[DHA];
  }
+if (options.abnuts[FAPU] == 0)
+ {
+ if (options.abnuts[OMEGA6] != 0) DV[OMEGA6] = options.abnuts[OMEGA6];
+ if (options.abnuts[LA] != 0) DV[LA] = options.abnuts[LA];
+ if (options.abnuts[AA] != 0) DV[AA] = options.abnuts[AA];
+ if (options.abnuts[OMEGA3] != 0) DV[OMEGA3] = options.abnuts[OMEGA3];
+ if (options.abnuts[ALA] != 0) DV[ALA] = options.abnuts[ALA];
+ if (options.abnuts[EPA] != 0) DV[EPA] = options.abnuts[EPA];
+ if (options.abnuts[DHA] != 0) DV[DHA] = options.abnuts[DHA];
+ temp = DV[OMEGA6] + DV[OMEGA3] - DV[FAPU];
+ DV[FAPU] += temp;
+ DV[FAMS] -= temp;
+ }
 if (options.pcsatfat > 0)
  {
  temp = DV[FASAT] + DV[FAMS];
@@ -482,23 +509,14 @@ if (options.abnuts[K] > 0) DV[K] = options.abnuts[K];
 if (options.abnuts[SE] > 0) DV[SE] = options.abnuts[SE]; 
 if (options.abnuts[NA] > 0) DV[NA] = options.abnuts[NA]; 
 if (options.abnuts[ZN] > 0) DV[ZN] = options.abnuts[ZN]; 
-DV[CHO_NONFIB] = DV[CHOCDF] - DV[FIBTG] < 0 ? 0 : DV[CHOCDF] - DV[FIBTG];
-if (DV[FAT] / DVBase[FAT] > DV[ENERC_KCAL] / DVBase[ENERC_KCAL])
+if (! options.abnuts[CHO_NONFIB]) DV[CHO_NONFIB] = DV[CHOCDF] - DV[FIBTG] < 0 ? 0 : DV[CHOCDF] - DV[FIBTG];
+else DV[CHO_NONFIB] = options.abnuts[CHO_NONFIB];
+temp = 1 - (DV[PROCNT] * options.protcalpergm + DV[CHOCDF] * options.carbcalpergm) / DV[ENERC_KCAL];
+if (temp > .3 && options.pcsatfat <= 0 && options.abnuts[FASAT] <= 0 && options.abnuts[FAMS] <= 0) 
  {
- temp = -1 + ((DV[FAT] / DVBase[FAT]) / (DV[ENERC_KCAL] / DVBase[ENERC_KCAL]));
- if (options.abnuts[FAPU] <= 0) DV[FAPU] += temp * adjla * DV[LA];
- if (options.abnuts[FAPU] <= 0) DV[FAPU] += temp * adjaa * DV[AA];
- if (options.abnuts[OMEGA6] <= 0) DV[OMEGA6] += temp * adjla * DV[LA];
- if (options.abnuts[OMEGA6] <= 0) DV[OMEGA6] += temp * adjaa * DV[AA];
- if (options.abnuts[FAMS] <= 0) DV[FAMS] -= temp * adjla * DV[LA];
- if (options.abnuts[FAMS] <= 0) DV[FAMS] -= temp * adjaa * DV[AA];
- if (options.abnuts[LA] <= 0) DV[LA] += temp * adjla * DV[LA];
- if (options.abnuts[AA] <= 0) DV[AA] += temp * adjaa * DV[AA];
- if (options.pcsatfat <= 0 && options.abnuts[FASAT] <= 0)
-  {
-  if (options.abnuts[FAMS] <= 0) DV[FAMS] -= temp * adjsat * DV[FASAT];
-  DV[FASAT] += temp * adjsat * DV[FASAT];
-  }
+ temp = DV[FAMS] + DV[FASAT];
+ if (temp * satfatpct > DV[FASAT]) DV[FASAT] = temp * satfatpct; 
+ DV[FAMS] = temp - DV[FASAT];
  }
 if ( options.pcsatfat > 0 || options.pcprotein > 0 || options.pccarb > 0 || options.fatprotratio || options.abnuts[PROCNT] > 0 || options.abnuts[CHOCDF] > 0 || options.abnuts[FIBTG] > 0 || options.abnuts[VITE]) options.custom = 1;
 else options.custom = 0;
@@ -584,6 +602,7 @@ if ( !options.locknuts[PROCNT] ) options.fatprotratio = 0;
 if ( !options.locknuts[CHOCDF] ) options.pccarb = 0;
 if ( !options.locknuts[FASAT] ) options.pcsatfat = 0;
 if ( removelock ) options.autocal = 0;
+if ( removelock ) options.n6hufaspec = 0;
 for ( i = 0; i < NUTRIENT_COUNT ; i++ ) if ( !options.locknuts[i] ) options.abnuts[i] = 0;
 if ( !options.locknuts[ENERC_KCAL] ) auto_cal(&DVBase[ENERC_KCAL],removelock ? 0 : 1);
 else auto_cal(&options.abnuts[ENERC_KCAL],removelock ? 0 : 1);
@@ -662,4 +681,104 @@ while (meal_ptr->next != NULL)
    }
   }
  }
+options.defanal = count - count % options.mealsperday;
+}
+
+void efa_dynamics()
+{
+int i;
+float temp, temp_la = 0, temp_aa = 0, extra_ala = 0, hufapct, p3, h3, h6, p6, o;
+float target = 60, laalaratio = 4.0, epadhalimit = 3.8;
+
+if (options.n6hufa != 0)
+ {
+ if (options.n6hufa < 15) options.n6hufa = 15;
+ if (options.n6hufa > 90) options.n6hufa = 90;
+ }
+
+options.abnuts[AA] = food_work.nutrient[AA];
+options.abnuts[LA] = food_work.nutrient[LA];
+options.abnuts[ALA] = food_work.nutrient[ALA];
+if (options.abnuts[ALA] < food_work.nutrient[LA] / laalaratio)
+ {
+ extra_ala = food_work.nutrient[LA] / laalaratio - options.abnuts[ALA];
+ options.abnuts[ALA] += extra_ala;
+ }
+p3 = 902 * (food_work.nutrient[SHORT3] + extra_ala) / food_work.nutrient[ENERC_KCAL];
+p6 = 902 * food_work.nutrient[SHORT6] / food_work.nutrient[ENERC_KCAL];
+h6 = 902 * food_work.nutrient[LONG6] / food_work.nutrient[ENERC_KCAL];
+o  = 902 * food_work.nutrient[FAT] * options.fattyacidfactor / food_work.nutrient[ENERC_KCAL] - food_work.nutrient[SHORT6] - food_work.nutrient[LONG6] - food_work.nutrient[SHORT3] - food_work.nutrient[LONG3];
+
+if (food_work.nutrient[EPA] > .02 || food_work.nutrient[DHA] > .02 || food_work.nutrient[AA] > .02) for (i = 1; i < 200000; i++)
+ {
+ temp = i * .001;
+ h3 = 902 * temp / food_work.nutrient[ENERC_KCAL];
+ hufapct = n6hufa(p3,p6,h3,h6,o);
+
+ if (options.n6hufa <= 0 && hufapct > target) continue;
+ if (options.n6hufa > 0 && hufapct > options.n6hufa) continue;
+
+ options.abnuts[OMEGA6] = food_work.nutrient[SHORT6] + food_work.nutrient[LONG6];
+ options.abnuts[OMEGA3] = food_work.nutrient[SHORT3] + extra_ala + temp;
+ temp -= food_work.nutrient[LONG3] - food_work.nutrient[EPA] - food_work.nutrient[DHA];
+ if (food_work.nutrient[EPA] > 0 || food_work.nutrient[DHA] > 0) temp = temp / (food_work.nutrient[EPA] + food_work.nutrient[DHA]);
+ options.abnuts[EPA] = food_work.nutrient[EPA] * temp;
+ options.abnuts[DHA] = food_work.nutrient[DHA] * temp;
+ if (options.abnuts[EPA] + options.abnuts[DHA] > epadhalimit)
+  {
+  temp = options.abnuts[EPA] + options.abnuts[DHA] - epadhalimit;
+  options.abnuts[EPA] -= options.abnuts[EPA] / (options.abnuts[EPA] + options.abnuts[DHA]) * temp;
+  options.abnuts[DHA] -= options.abnuts[DHA] / (options.abnuts[EPA] + options.abnuts[DHA]) * temp;
+  options.abnuts[OMEGA3] -= temp;
+  h3 -= 902 * temp / food_work.nutrient[ENERC_KCAL];
+  for (i = 1; i < 200000; i++)
+   {
+   temp_la = p6 * i * .00001;
+   temp_aa = h6 * i * .00001;
+   hufapct = n6hufa(p3,p6-temp_la,h3,h6-temp_aa,o);
+   if (options.n6hufa <= 0 && hufapct < target) break;
+   if (options.n6hufa > 0 && hufapct < options.n6hufa) break;
+   }
+  options.abnuts[LA] -= temp_la / 902 * food_work.nutrient[ENERC_KCAL];
+  options.abnuts[AA] -= temp_aa / 902 * food_work.nutrient[ENERC_KCAL];
+  options.abnuts[OMEGA6] -= (temp_la + temp_aa) / 902 * food_work.nutrient[ENERC_KCAL];
+  }
+ if (options.abnuts[OMEGA6] == 0) options.abnuts[OMEGA6] = .01;
+ if (options.abnuts[LA] == 0) options.abnuts[LA] = .01;
+ if (options.abnuts[AA] == 0) options.abnuts[AA] = .01;
+ if (options.abnuts[OMEGA3] == 0) options.abnuts[OMEGA3] = .01;
+ if (options.abnuts[ALA] == 0) options.abnuts[ALA] = .01;
+ if (options.abnuts[EPA] == 0) options.abnuts[EPA] = .01;
+ if (options.abnuts[DHA] == 0) options.abnuts[DHA] = .01;
+ return;
+ }
+else
+ {
+ h3 = 902 * food_work.nutrient[LONG3] / food_work.nutrient[ENERC_KCAL];
+ for (i = 1; i < 200000; i++)
+  {
+  temp = i * .001;
+
+  p3 = 902 * temp / food_work.nutrient[ENERC_KCAL];
+  o  = 902 * (food_work.nutrient[FASAT] + food_work.nutrient[FAMS]) / food_work.nutrient[ENERC_KCAL];
+  hufapct = n6hufa(p3,p6,h3,h6,o);
+
+  if (options.n6hufa <= 0 && hufapct > target) continue;
+  if (options.n6hufa > 0 && hufapct > options.n6hufa) continue;
+  options.abnuts[OMEGA6] = food_work.nutrient[SHORT6] + food_work.nutrient[LONG6];
+  options.abnuts[OMEGA3] = food_work.nutrient[LONG3] + temp;
+  temp -= food_work.nutrient[SHORT3] - food_work.nutrient[ALA];
+  options.abnuts[ALA] = temp;
+  options.abnuts[EPA] = .01;
+  options.abnuts[DHA] = .01;
+  if (options.abnuts[OMEGA6] == 0) options.abnuts[OMEGA6] = .01;
+  if (options.abnuts[LA] == 0) options.abnuts[LA] = .01;
+  if (options.abnuts[AA] == 0) options.abnuts[AA] = .01;
+  if (options.abnuts[OMEGA3] == 0) options.abnuts[OMEGA3] = .01;
+  if (options.abnuts[ALA] == 0) options.abnuts[ALA] = .01;
+  if (options.abnuts[EPA] == 0) options.abnuts[EPA] = .01;
+  if (options.abnuts[DHA] == 0) options.abnuts[DHA] = .01;
+  return;
+  }
+ }
 }
